classdef cone
    %UNTITLED Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        g
        %
        rho
        R
        H
        %
        m
        Iy
        Iz
        T2
        i
        j
        %
        
    end
    
    methods
        function obj = cone(data)
            %UNTITLED Construct an instance of this class
            %   Detailed explanation goes here
            SIunits = containers.Map;
            SIunits('Hz')     =1;
            SIunits('m')      =1;
            SIunits('cm')     =0.01;
            SIunits('g')      =1/1000;
            SIunits('m/s^2')  =1;
            SIunits('m/s')    =1;
            SIunits('1/(m/s)')=1;
            SIunits('N/m')    =1;
            SIunits('kg/m^3') =1;
            SIunits('%')      =0.01;
            SIunits('')       =1;
            SIunits('°')      =pi/180;

            obj.R    = data({'R'    },:).number*SIunits(data({'R'    },:).unit{1});
            obj.H    = data({'H'    },:).number*SIunits(data({'H'    },:).unit{1});
            obj.rho  = data({'rho'  },:).number*SIunits(data({'rho'  },:).unit{1});
            obj.g    = data({'g'    },:).number*SIunits(data({'g'    },:).unit{1});

            obj.m    = obj.rho*pi*obj.R^2*obj.H/ 3;
            obj.Iz   = obj.m*obj.R^2/10;
            obj.Iy   = obj.m*(3*obj.R^2+12*obj.H^2)/20;

            obj.i    = obj.Iz/obj.Iy;
            obj.j    = obj.H^2*obj.m/obj.Iy;
            
            obj.T2   = 2*obj.H/(3*obj.g);
            
        end
        
        function dydt = odeRhs(obj, t, y)
            %METHOD dydt delivers rhs of eom
            %   eom for gyroscope

            % angles
            phi   = y(1:3,1);
            % angular velocities
            omega = y(4:6,1);
            
            c = [cos(phi(1,1)),cos(phi(2,1))];
            s = [sin(phi(1,1)),sin(phi(2,1))];
            
            % M^(-1)*G, M^(-1)*P
            %[mig, mip] = obj.MiG(phi);
            
            dydt(1:3,1) =  omega;
            %dydt(4:6,1) = -omega(3)*mig*omega+mip;
            dydt(4:6,1) = ...
            [-s(1)*obj.i*omega(2)*omega(3)-(c(1)*s(1)*obj.i-c(1)*s(1))*omega(2)^2;...
             (obj.i*omega(1)*omega(3)+(c(1)*obj.i-2*c(1))*omega(1)*omega(2))/s(1);...
             -(c(1)*obj.i*omega(1)*omega(3)+(c(1)^2*obj.i-c(1)^2-1)*omega(1)*omega(2))/s(1)]...
            + ...
            [s(1)*obj.j;
               0;
               0];            
        end
                
        function trajectory(obj,t,y)

            % local coordinates
            tT = transpose(t);
            yT = transpose(y);

            phi   = yT(1:3,:);
            omega = yT(4:6,:);

            xyz = zeros(3,length(t));
            for i=1:length(t)
                s = [sin(phi(1,i)), sin(phi(2,i)), sin(phi(3,i))];
                c = [cos(phi(1,i)), cos(phi(2,i)), cos(phi(3,i))];
                xyz(1:3,i) = 1.1*[s(1)*c(2); ...
                                  s(1)*s(2); ...
                                  c(1)];
            end
                        
            tiledlayout(2,2);
            % plot
            nexttile;
            plot(t,phi(1,:),'-',t,phi(2,:),'-');
            grid on;
            title('Kipp-Winkel');
            legend({'φ_1(t)','φ_2(t)'},'Location','southwest')
            xlabel('\tau →') 
            ylabel('φ →') 
            
            % Next plot
            nexttile;
            plot(t,omega(1,:),'-',t,omega(2,:),'-');
            grid on;
            title('Winkelgeschwindigkeiten');
            legend({'\omega_1(t)','\omega_2(t)'},'Location','southwest')
            xlabel('\tau →') 
            ylabel('\omega →') 
            
            nexttile;
            [x,y,z] = sphere;      %# Makes a 21-by-21 point sphere
            x = x(11:end,:);       %# Keep top 11 x points
            y = y(11:end,:);       %# Keep top 11 y points
            z = z(11:end,:);       %# Keep top 11 z points
            hold on;
            hSurface=surf(x,y,z);
            set(hSurface,'FaceColor',[0.6 0.6 0.6], ...
               'FaceAlpha',0.5,'FaceLighting','gouraud','EdgeColor','none')
            plot3(xyz(1,:),xyz(2,:),xyz(3,:));
            xlabel('x_1 →') 
            ylabel('x_2 →') 
            zlabel('x_3 →') 
            hold off;

            nexttile;
            plot(omega(1,:),phi(1,:),'-',omega(2,:),phi(2,:),'-');
            grid on;
            title('Phasen-Diagramme');
            legend({'φ_1(t)','φ_2(t)'},'Location','southwest')
            xlabel('\omega →') 
            ylabel('φ →') 

        end
        
    end
end

